/** 3DGPL *************************************************\
 * (MSDOS, i386+, VGA, DJGPPv1 or v2)                     *
 * Header for hardware specific stuff.                    *
 *                                                        *
 * Defines:                                               *
 *  HW_init_screen           Opening output surface;      *
 *  HW_blit                  Colourmap onto the screen;   *
 *  HW_close_screen          Closing output;              *
 *                                                        *
 *  HW_init_event_loop       Runing for events;           *
 *  HW_error                 Quiting with a message;      *
 *  HW_close_event_loop      Quiting running.             *
 *                                                        *
 * (c) 1995-98 Sergei Savchenko, (savs@cs.mcgill.ca)      *
\**********************************************************/

#include "../hardware/hardware.h"           /* all the defenitions */
#include "../colour/colour.h"               /* CL_ stuff */
#include "../graphics/graphics.h"           /* G_ stuff */
#if (DJGPP==2)
 #include <sys/nearptr.h>                   /* v2 __djgpp stuff */
#endif
#include <pc.h>                             /* kbhit() etc. */
#include <stdlib.h>                         /* exit */
#include <stdarg.h>                         /* va arg stuff */
#include <stdio.h>                          /* vfprintf */

HW_pixel *HW_vram;                          /* points video ram */
int HW_running;                             /* event loop running? */

/**********************************************************\
 * Opening output and setting the palette.                *
\**********************************************************/

#define HW_TOTAL_COLOUR_CELLS 256           /* can't do more colours */

#if defined(_CI_)
void HW_init_screen(char *display_name,
                    char *screen_name
                   )
{
 int i;
 unsigned char r,g,b;

#if DJGPP == 1
 HW_vram=(unsigned char*)0xd0000000;        /* address for video memory */
#endif
#if DJGPP == 2
 HW_vram=(unsigned char*)0xa0000+__djgpp_conventional_base;
#endif

 asm("movb $0x13,%%al":::"%al");            /* %al mode number */
 asm("xorb %%ah,%%ah":::"%ah");             /* function #0 */
 asm("int  $0x10");

 if(CL_colours->cl_no_colours>HW_TOTAL_COLOUR_CELLS)
  HW_error("(Hardware) Can't realize that big a palette.\n");

 for(i=0;i<CL_colours->cl_no_colours;i++)
 {
  r=CL_colours->cl_colours[i].cl_red>>2;
  g=CL_colours->cl_colours[i].cl_green>>2;
  b=CL_colours->cl_colours[i].cl_blue>>2;

  asm("movb %0,%%dh":: "g" (r):"%dh");      /* Red */
  asm("movb %0,%%ch":: "g" (g):"%ch");      /* Green */
  asm("movb %0,%%cl":: "g" (b):"%cl");      /* Blue */
  asm("movw %0,%%ebx":: "g" (i):"%ebx");    /* Colour to set */
  asm("movw $0x1010,%%ax":::"%ax");         /* function 0x10 subfun 0x10 */
  asm("int  $0x10");
 }
}
#endif
#if defined(_RGB_)
void HW_init_screen(char *display_name,
                    char *screen_name
                   )
{
 int i;
 unsigned char r,g,b;

#if DJGPP == 1
 HW_vram=(unsigned char*)0xd0000000;        /* address for video memory */
#endif
#if DJGPP == 2
 HW_vram=(unsigned char*)0xa0000+__djgpp_conventional_base;
#endif

 asm("movb $0x13,%%al":::"%al");            /* %al mode number */
 asm("xorb %%ah,%%ah":::"%ah");             /* function #0 */
 asm("int  $0x10");

 for(i=0;i<HW_TOTAL_COLOUR_CELLS;i++)
 {
  r=CL_red(i)<<3;
  g=CL_green(i)<<4;
  b=CL_blue(i)<<3;

  asm("movb %0,%%dh":: "g" (r):"%dh");      /* Red */
  asm("movb %0,%%ch":: "g" (g):"%ch");      /* Green */
  asm("movb %0,%%cl":: "g" (b):"%cl");      /* Blue */
  asm("movw %0,%%ebx":: "g" (i):"%ebx");    /* Colour to set */
  asm("movw $0x1010,%%ax":::"%ax");         /* function 0x10 subfun 0x10 */
  asm("int  $0x10");
 }
}
#endif

/**********************************************************\
 * Colormap onto the screen.                              *
\**********************************************************/

void HW_blit(void)
{
#if (DJGPP==2)
 __djgpp_nearptr_enable();
#endif
 HW_copy_pixel(G_c_buffer,HW_vram,G_c_buffer_size);
#if (DJGPP==2)
 __djgpp_nearptr_disable();
#endif
}

/**********************************************************\
 * Closing output.                                        *
\**********************************************************/

void HW_close_screen(void)
{
}

/**********************************************************\
 * Running the event loop.                                *
\**********************************************************/

void HW_init_event_loop(void (*application_main)(void),
                        void (*application_key_handler)(int key_code)
                       )
{
 HW_running=1;

 while(HW_running==1)                       /* still running? hmm */
 {
  if(kbhit()) application_key_handler(getkey());
  application_main();
 }
}

/**********************************************************\
 * Stoping with a message.                                *
\**********************************************************/

void HW_error(char *s,...)
{
 va_list lst;

 va_start(lst,s);
 vfprintf(stderr,s,lst);                    /* printing a message */
 va_end(lst);
 HW_close_event_loop();                     /* quiting processing */
 exit(0);
}

/**********************************************************\
 * Stoping the event loop.                                *
\**********************************************************/

void HW_close_event_loop(void)
{
 HW_running=0;                              /* that's it */
}

/**********************************************************/
