#if !defined(_8BPP_)
 #error This hardware interface can handle it at just _8BPP_.
#endif
#if ((!defined(_MONO_)) || defined(_STEREO_))
 #error This hardware interface can handle just _MONO_.
#endif
#if !defined(_HARDWARE_H_)
#define _HARDWARE_H_

/** 3DGPL *************************************************\
 * (MSDOS, i386+, VGA, WATCOM C)                          *
 * Header for hardware specific stuff.                    *
 *                                                        *
 * Ifdefs:                                                *
 *  _CI_                     Colour/Intensity model;      *
 *  _RGB_                    RGB model;                   *
 *  _8BPP_                   8 bit per pixel map;         *
 *  _MONO_                   Regular image.               *
 *                                                        *
 * Files:                                                 *
 *  hardware.c               Hardware specific stuff.     *
 *                                                        *
 * (c) 1995-98 Sergei Savchenko, (savs@cs.mcgill.ca)      *
\**********************************************************/

void HW_set_char(char *dst,int lng,char val);
#pragma aux HW_set_char=  \
 "cld"                    \
 "rep stosb"              \
 parm [edi] [ecx] [al];
void HW_copy_char(char *src,char *dst,int lng);
#pragma aux HW_copy_char= \
 "cld"                    \
 "rep movsb"              \
 parm [esi] [edi] [ecx];

void HW_set_int(int *dst,int lng,int val);
#pragma aux HW_set_int=   \
 "cld"                    \
 "rep stosd"              \
 parm [edi] [ecx] [eax];
void HW_copy_int(int *src,int *dst,int lng);
#pragma aux HW_copy_int=  \
 "cld"                    \
 "rep movsd"              \
 parm [esi] [edi] [ecx];

typedef char HW_8_bit;                      /* compiler/machine independent */
#define HW_set_8_bit(dst,lng,val) HW_set_char(dst,lng,val)
#define HW_copy_8_bit(src,dst,lng) HW_copy_char(src,dst,lng)

typedef short HW_16_bit;
void HW_set_16_bit(HW_16_bit *dst,int lng,HW_16_bit val);
#pragma aux HW_set_16_bit=   \
 "cld"                       \
 "rep stosw"                 \
 parm [edi] [ecx] [ax];
void HW_copy_16_bit(HW_16_bit *src,HW_16_bit *dst,int lng);
#pragma aux HW_copy_16_bit=  \
 "cld"                       \
 "rep movsw"                 \
 parm [esi] [edi] [ecx];

typedef int HW_32_bit;
#define HW_set_32_bit(dst,lng,val)  HW_set_int(dst,lng,val)
#define HW_copy_32_bit(src,dst,lng) HW_copy_int(src,dst,lng)

#if defined(_8BPP_)
typedef HW_8_bit HW_pixel;
#define HW_set_pixel(dst,lng,val) HW_set_8_bit(dst,lng,val)
#define HW_copy_pixel(src,dst,lng) HW_copy_8_bit(src,dst,lng)
#endif

#define HW_SCREEN_X_SIZE 320
#define HW_SCREEN_Y_SIZE 200                /* number of pixels total */
#if ((HW_SCREEN_X_SIZE != 320) || (HW_SCREEN_Y_SIZE != 200))
 #error This hardware interface only supports 320x200 window.
#endif
#if defined(_MONO_)
 #define HW_SCREEN_LINE_SIZE HW_SCREEN_X_SIZE
#endif
#define HW_SCREEN_X_MAX    (HW_SCREEN_X_SIZE-1)
#define HW_SCREEN_Y_MAX    (HW_SCREEN_Y_SIZE-1)
#define HW_SCREEN_X_CENTRE (HW_SCREEN_X_SIZE/2)
#define HW_SCREEN_Y_CENTRE (HW_SCREEN_Y_SIZE/2)

#define HW_KEY_ARROW_LEFT  75
#define HW_KEY_ARROW_RIGHT 77
#define HW_KEY_ARROW_UP    72
#define HW_KEY_ARROW_DOWN  80
#define HW_KEY_PLUS        43
#define HW_KEY_MINUS       45
#define HW_KEY_ENTER       13
#define HW_KEY_SPACE       32
#define HW_KEY_TAB          9               /* all i can think of */

void HW_init_screen(char *display_name,
                    char *screen_name
                   );
void HW_blit(void);
void HW_close_screen(void);
void HW_init_event_loop(void (*application_main)(void),
                        void (*application_key_handler)(int key_code)
                       );
void HW_error(char *s,...);
void HW_close_event_loop(void);

/**********************************************************/

#endif
