/** 3DGPL *************************************************\
 * (Macintosh, 8/16bit deep modes)                        *
 * Header for hardware specific stuff.                    *
 *                                                        *
 * Ifdefs:                                                *
 *  _CI_                     Colour/Intensity model;      *
 *  _RGB_                    RGB model;                   *
 *  _8BPP_                   8 bit per pixel;             *
 *  _16BPP_                  16 bit per pixel;            *
 *  _32BPP_                  32 bit per pixel.            *
 *                                                        *
 * Defines:                                               *
 *  HW_set_int               Should be a macro;           *
 *  HW_set_16_bit            And this one.                *
 *                                                        *
 *  HW_init_screen           Opening output surface;      *
 *  HW_blit                  Colourmap onto the screen;   *
 *  HW_close_screen          Closing output;              *
 *                                                        *
 *  HW_init_event_loop       Runiing for events;          *
 *  HW_error                 Quiting with a message;      *
 *  HW_close_event_loop      Quiting running.             *
 *                                                        *
 * (c) 1995-98 Sergei Savchenko, (savs@cs.mcgill.ca)      *
\**********************************************************/

#include "../hardware/hardware.h"           /* harware dependent stuff */
#include "../colour/colour.h"               /* CL_ stuff */
#include "../graphics/graphics.h"           /* G_ stuff */
#include <stdarg.h>                         /* var args */
#include <stdlib.h>                         /* exit */
#include <QuickDraw.h>                      /* PixMap etc */
#include <WindowMgr.h>                      /* WindowPtr etc */
#include <EventMgr.h>                       /* Events etc */

WindowPtr HW_Window;                        /* poins a window */
PixMap HW_PixMap;                           /* colourmap */
Rect HW_Rect;

int HW_running;                             /* event loop running? */

/**********************************************************\
 * Implementations for fast memory fills.                 *
\**********************************************************/

void HW_set_int(int *d,long l,int v)
{
 long i;

 for(i=0;i<l;i++) *d++=v;
}

/**********************************************************\
 * Implementations for another fast memory fills.         *
\**********************************************************/

void HW_set_16_bit(HW_16_bit *d,long l,HW_16_bit v)
{
 long i;

 for(i=0;i<l;i++) *d++=v;
}

/**********************************************************\
 * Creating a window.                                     *
\**********************************************************/

void HW_init_screen(char *display_name,
                    char *screen_name,
{

 InitGraf(&qd.thePort);
 InitWindows();

 SetRect(&HW_Rect,0,0,HW_SCREEN_X_SIZE,HW_SCREEN_Y_SIZE);
 HW_Window=NewWindow(0L,&HW_Rect,0L,true,0,(WindowPtr)-1L,false,0L);

 G_c_buffer=(short*)malloc(sizeof(short)*HW_SCREEN_X_SIZE*HW_SCREEN_Y_SIZE);

 HW_PixMap.bounds=HW_Rect;
 HW_PixMap.rowBytes=0x8000 | (HW_SCREEN_X_SIZE*sizeof(short));
 HW_PixMap.baseAddr=(char*)G_c_buffer;
 HW_PixMap.hRes=72<<16;
 HW_PixMap.vRes=72<<16;
 HW_PixMap.pmVersion=0;
 HW_PixMap.packType=0;
 HW_PixMap.packSize=sizeof(short)*HW_SCREEN_X_SIZE*HW_SCREEN_Y_SIZE;
 HW_PixMap.planeBytes=sizeof(short)*HW_SCREEN_X_SIZE*HW_SCREEN_Y_SIZE;
 HW_PixMap.pmReserved=0;
 HW_PixMap.pmTable=nil;
 HW_PixMap.pixelType=RGBDirect;
 HW_PixMap.cmpCount=3;
 HW_PixMap.pixelSize=16;
 HW_PixMap.cmpSize=5;

 ShowWindow(HW_Window);
 SetPort(HW_Window);
}

/**********************************************************\
 * Blitting a bitmap into the allocated window.           *
\**********************************************************/

void HW_blit(void)
{
 CopyBits((BitMap*)&HW_PixMap,&HW_Window->portBits,&HW_Rect,&HW_Rect,
          srcCopy,nil
         );
}

/**********************************************************\
 * Deallocating a window.                                 *
\**********************************************************/

void HW_close_screen(void)
{
 CloseWindow(HW_Window);
}

/**********************************************************\
 * The main event loop.                                   *
\**********************************************************/

void HW_init_event_loop(void (*application_main)(void),
                        void (*application_key_handler)(int key_code)
                       )
{
 EventRecord anEvent;

 HW_running=1;
 while(HW_running)
 {
  if(GetNextEvent(everyEvent, &anEvent))
   switch(anEvent.what)
   {
    case keyDown: application_key_handler(anEvent.message); break;
    default: break;
   }
  application_main();
 }
}

/**********************************************************\
 * Quitng with a message.                                 *
\**********************************************************/

#define HW_MAX_ERROR_MESSAGE 256

void HW_error(char *s,...)
{
 char str[HW_MAX_ERROR_MESSAGE];
 va_list lst;

 va_start(lst,s);
 vsprintf(str,s,lst);                       /* forming the message */
 va_end(lst);

 HW_close_event_loop();
 exit(0);                                   /* above might not be enough */
}

/**********************************************************\
 * Quiting the event loop.                                *
\**********************************************************/

void HW_close_event_loop(void)
{
 HW_running=0;
}

/**********************************************************/
